package cellular;

import java.io.OutputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import cellular.k.OutputWriteable;

/**
 * <p> </p>
 * <p> </p>
 * <p> Copyright (c) 2003</p>
 * <p> WoCoWa</p>
 * @author Wouter Koolen-Wijkstra
 * @version 1.0
 */

public class Evolution implements OutputWriteable
{
	final int w;
	final int h;
	final int border;
	final int [][] field;
	final Rule r;

    public Evolution(int h, Rule r, int initial)
    {
		this.h = h;
		this.w = initial + h * 2 * r.getContextExtent();
		this.r = r;
		this.border = r.getContextExtent();
		field = new int [h][w+2*border];

		field[0] = new int [initial];

		// user demands 'initial' length, so first and last are
		// forcefully non-zero (otherwise the segment would be shorter)
		field[0][0] = 1 + (int)(Math.random() * (r.getSymbols()-1));
		for (int i = 1; i < field[0].length-1; i++)
		{
			field[0][i] =  (int)(Math.random() * r.getSymbols());
		}
		field[0][field[0].length-1] = 1 + (int)(Math.random() * (r.getSymbols()-1));

		for (int y = 1; y < h; y++)
		{
			int [] parent =     field[y-1];
			int [] current =    new int[parent.length + 2 * border];

			for (int x = 0; x < current.length; x++)
			{
				current[x] = r.evaluate(parent, x-border);
			}

		    field[y] = current;
		}
    }

	public int get(int x, int y)
	{
		assert (x >= 0 && x < w);
		assert (y >= 0 && y < h);

		int xr = x - w/2+border*y;

		if (xr < 0 || xr >= field[y].length) return 0;

		return field[y][xr];
	}

	public int getWidth()
	{
		return w;
	}

	public int getHeight()
	{
		return h;
	}

	public Rule getRule()
	{
		return r;
	}


	public void write(OutputStream os) throws IOException
	{
		DataOutputStream s = new DataOutputStream(os);

		s.writeInt(h);
		for (int i = 0; i < field.length; i++)
		for (int j = 0; j < field[i].length; j++)
		{
			s.writeByte(field[i][j]);
		}
	}

}