package cellular;

import java.math.BigDecimal;
import java.math.BigInteger;

/**
 * <p> </p>
 * <p> </p>
 * <p> Copyright (c) 2003</p>
 * <p> WoCoWa</p>
 * @author Wouter Koolen-Wijkstra
 * @version 1.0
 */

public class Rule
{
	final int symbols;
	final int extent;

	final int [] rule;


    public Rule(int symbols, int context)
    {
		assert context % 2 == 1;
		this.symbols =  symbols;
		this.extent =   (context-1)/2;

		// compute symbol ^ context
		int nrules = 1;
		// smarter: use bisection exponentiation
		for (int i = 0; i < context; i++)
		{
			nrules *= symbols;
		}

		rule = new int [nrules];

		rule[0] = 0; // reproduce zeros.

		for (int i = 1; i < nrules; i++)
		{
			rule[i] = (int)(Math.random()*symbols);
		}
    }

    public Rule(int symbols, int context, BigInteger nrule)
    {
		assert context % 2 == 1;
		this.symbols =  symbols;
		this.extent =   (context-1)/2;

		// compute symbol ^ context
		int nrules = 1;
		// smarter: use bisection exponentiation
		for (int i = 0; i < context; i++)
		{
			nrules *= symbols;
		}

		rule = new int [nrules];
		BigInteger radix = BigInteger.valueOf(symbols);

		for (int i = 0; i < rule.length; i++)
		{
			BigInteger [] divrem = nrule.divideAndRemainder(radix);
			rule[i] = divrem[1].intValue();
			nrule = divrem[0];
		}
		assert(nrule.equals(BigInteger.ZERO));
    }


	public int getSymbols()
	{
		return symbols;
	}

	public int evaluate(int [] context, int pos)
	{
		// compute context[pos-extent] * symbols^context ... context[pos+extent] * symbols^0
		int nrule = 0;
		for (int i = pos-extent ; i <= pos+extent; i++)
		{
			// two-step horners evaluation method
			nrule= symbols * nrule;
			if (i >= 0 && i < context.length) nrule += context[i];
		}

		return rule[nrule];
	}


	public int getContextExtent()
	{
		return extent;
	}

	public int getNRules()
	{
		return rule.length;
	}

	public BigInteger getRuleNumber()
	{
		BigInteger n =      BigInteger.ZERO;
		BigInteger radix =  BigInteger.valueOf(symbols);

		for (int i = rule.length-1; i >= 0; i--)
		{
			n = radix.multiply(n).add(BigInteger.valueOf(rule[i]));;
		}
		return n;
	}
}